function  seg = sky_seg(im, model_sunny, model_cloudy)
%sky_seg - Image Smoothing via L0 Gradient Minimization
%   seg = sky_seg(im, model) segment sky region from input
%   image im, with trained random forest model model_sunny and model_cloudy.
%
%   Paras: 
%   @im   : Input double image, should be color images.
%   @model_sunny:  a learn sunny sky patch model of random forest.
%   @model_cloudy:  a learn cloudy sky patch model of random forest.
%   @seg  : a binay map labeling sky (non-sky) region as 1 (0).
%   Example
%   ==========
%   im  = im2double(imread('ICCV2013.jpg'));
%   seg = sky_seg(im, model_sunny, model_cloudy);
%   figure, imshow(seg) 

[ori_H, ori_W, ~] = size(im);
wSzie = 400;

im = imresize(im,[wSzie,wSzie]);
[map1,map2] = sky_probability(im, model_cloudy, model_sunny);
mapc = max(max(map2,map1),0);
[L1,L2,~] = size(im);
N = L1*L2;
h = GCO_Create(N,2);             % Create new object with NumSites=N, NumLabels=2
T =   int32(max(round(5*[abs(mapc(:)-1),abs(mapc(:))]),0));
 
GCO_SetDataCost(h,T');   % Site  3   prefers label 3
GCO_SetSmoothCost(h,[0 1;1 0]);   % 

E = edges4connected(L1,L2);
m1 = im(:,:,1); m1 = m1(:);
m2 = im(:,:,2); m2 = m2(:);
m3 = im(:,:,3); m3 = m3(:);

V = exp(-100*sqrt(abs(m1(E(:,1))-m1(E(:,2))).^2 + abs(m2(E(:,1))-m2(E(:,2))).^2  +  abs(m3(E(:,1))-m3(E(:,2))).^2)) ;
V = max(round(V*1000),0);
A = sparse(E(:,1),E(:,2),V,N,N,4*N);


GCO_SetNeighbors(h,A);
GCO_Expansion(h);  
u= GCO_GetLabeling(h);

seg = double(reshape(u,[400,400]));
seg = 2 - imresize(seg, [ori_H, ori_W], 'nearest');

end

function [map1,map2] = sky_probability(im, model_cloud, model_sunny)

    patchSize = 15;
    gridSpacing = 3;
    threshold = 1;
    [im_h, im_w, ~] = size(im);       
    remX = mod(im_w-patchSize,gridSpacing);
    offsetX = floor(remX/2)+1;
    remY = mod(im_h-patchSize,gridSpacing);
    offsetY = floor(remY/2)+1;
    [gridX,gridY] = meshgrid(offsetX:gridSpacing:im_w-patchSize+1, offsetY:gridSpacing:im_h-patchSize+1);

    hsv_arr = sp_find_HSV_grid(im, gridX, gridY, patchSize);
    sift_arr = sp_find_sift_grid(rgb2gray(im), gridX, gridY, patchSize);
    [sift_arr, siftlen] = sp_normalize_sift(sift_arr, threshold);

    X = [sift_arr,hsv_arr];


    Y_hat1 = regRF_predict(X,model_cloud); 
    map1 = sky_cof_map(im, gridX, gridY, patchSize, Y_hat1);

    Y_hat2 = regRF_predict(X,model_sunny); 
    map2 = sky_cof_map(im, gridX, gridY, patchSize, Y_hat2);

end
